// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// GPU initialization and shutdown
// ===============================

#pragma once

namespace hawaii {
namespace GPU {

// initialize all available GPUs with the following preferences:
// - be able to map page-locked host memory into the device address space
// - yield the controlling CPU thread when synchronizing (instead of potentially having it spin)
// - enable peer-to-peer communication with all other GPUs
// user note: In case of "cudaErrorSetOnActiveProcess" you are calling this too late within your application.
// developer note: If this wasn't a static library, this could be done automatically in the c'tor of a singleton.
void init() ;

// clear libHawaii's GPU memory pools: Calling this at the end of your application avoids a run-time error occurring 
//                                     because OpenCV's destructors might already be unloaded before the pools' content 
//                                     is deleted.
void reset() ;

} } // namespaces "hawaii::GPU"
