// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// processing step to save images to disk
// ======================================

#pragma once

#include "hawaii/flow/step.h"
#include "hawaii/flow/itemNamedImages.h"

namespace hawaii {
namespace IO {

// save image with filename generated from pattern, label and index
void saveIndexed( const cv::Mat     image,
                  const std::string pattern,
                  const std::string label,
                  const uint64_t    index   ) ;

// processing step to save images to disk
class StepSave : public hawaii::flow::StepTemplate< hawaii::flow::ItemNamedImages,
                                                    hawaii::flow::ItemBase        > {
	// c'tor with the options below
	public:
	StepSave( const std::string filePatternArg,
	          const uint64_t    indexStartArg  = 0 ) :
		hawaii::flow::StepTemplate< hawaii::flow::ItemNamedImages,
		                            hawaii::flow::ItemBase        >( "save images to disk",
		                                                             1, std::vector< int >(),
		                                                             false, true ),
		filePattern( filePatternArg ),
		indexStart( indexStartArg ) {
	}
	
	// filename pattern for use with "sprintf"
	// user note: must contain exactly 2 variables, first "%s" for the image instance name within the work item, second 
	//            an unsigned 64-bit integer with or without leading zeros and a user-defined number of digits, e.g. 
	//            "myResults/%s%010llu.png".
	protected:
	const std::string filePattern ;
	
	// offset to work item's serial number
	protected:
	const uint64_t indexStart ;
	
	// implement inherited purely-virtual method to save all images within the item
	public:
	virtual void processImpl( const SrcPtr& srcPtr,
	                                DstPtr& dstPtr ) override ;
} ;

} } // namespaces "hawaii::IO"
