// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// various small helpers
// =====================

#pragma once

#include "hawaii/common/ostream.h"
#include "hawaii/common/cPlusPlus11.h"
#include <opencv2/highgui/highgui.hpp>

// print a variable or show an image including its name without having to type it twice
#define HAWAII_PRINT(  var   ) { hawaii::cout << #var " = " << ( var ) << hawaii::endl ; }
#define HAWAII_IMSHOW( cvMat ) { cv::imshow( #cvMat, cvMat ) ; }

namespace hawaii {

// delete only non-null objects/arrays and - if possible - set pointer to null afterwards
template< typename T > inline void deleteObject( T const * __restrict__       & objPtr ) { if( objPtr ) { delete   objPtr ; objPtr = nullptr ; } }
template< typename T > inline void deleteObject( T const * __restrict__ const & objPtr ) { if( objPtr ) { delete   objPtr ;                    } }
template< typename T > inline void deleteArray(  T const * __restrict__       & arrPtr ) { if( arrPtr ) { delete[] arrPtr ; arrPtr = nullptr ; } }
template< typename T > inline void deleteArray(  T const * __restrict__ const & arrPtr ) { if( arrPtr ) { delete[] arrPtr ;                    } }

} // namespace "hawaii"
