// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// base classes for work items being passed through flow components
// ================================================================

#include "hawaii/flow/item.h"

namespace hawaii {
namespace flow {

// "ItemBase": base work item type
// -------------------------------

// signal to stop processing
const ItemBase::Ptr itemFinalPtr( new ItemBase( ItemBase::serialNumberFinal ) ) ;

// partitioning methods: overriding implementations only needed if partitioning is actually used
void ItemBase::split(       ItemBase::PtrVec&      partsBase,
                      const std::vector< double >& ratios,
                      const PartInfoBase&          partInfoBase ) {
	
	// regular processing
	if( HAWAII_LIKELY( !this->isFinal() ) ) {
		HAWAII_ERROR( "not implemented in derived work item type" ) ;
		(void)partsBase    ;
		(void)ratios       ;
		(void)partInfoBase ;
		
	// distribute final items
	} else {
		partsBase = ItemBase::PtrVec( ratios.size(), itemFinalPtr ) ;
	}
}
void ItemBase::merge( const ItemBase::PtrVec& partsBase,
                            ItemBase::Ptr&    wholeBase,
                      const PartInfoBase&     partInfoBase ) {
	
	// regular processing
	bool isFinal = false ;
	for( const auto& partBase : partsBase ) { isFinal |= partBase->isFinal() ; }
	if( HAWAII_LIKELY( !isFinal ) ) {
		HAWAII_ERROR( "not implemented in derived work item type" ) ;
		(void)partsBase    ;
		(void)wholeBase    ;
		(void)partInfoBase ;
		
	// return final item
	} else {
		wholeBase = itemFinalPtr ;
	}
}

} } // namespaces "hawaii::flow"
